@@ -12,10 +12,10 @@ To add a new check:
 
     1. Create a function that returns a true like, printable object for
        failure, or ``None`` for success.
-    2. Add that function to ``CHECKS``.
+    2. Add that function to ``captainhook.checkers.ALL_CHECKS``.
     3. Add a corresponding line (the name of the function) to your tox.ini file
        with 'on' (run this check) or 'off' (don't). The default behaviour is to
-       run all checks defined in ``CHECKS``.
+       run all checks defined in ``ALL_CHECKS``.
 """
 try:
     import ConfigParser as configparser
@@ -24,32 +24,34 @@ except ImportError:
     import configparser
 
 from contextlib import contextmanager
+import importlib
 import os.path
+import pkgutil
 import sys
-from subprocess import Popen, PIPE
+import types
+
+path = os.path.dirname(os.path.abspath(__file__))
+sys.path.append(path)
+
+import checkers
+from checkers.utils import bash
 
 
-class bash(object):
-    "This is lower class because it is intended to be used as a method."
+def checks():
+    """
+    An iterator of valid checks that are in the installed checkers package.
 
-    def __init__(self, cmd):
-        """
-        TODO: Release this as a separate library!
-        """
-        self.p = None
-        self.output = None
-        self.bash(cmd)
+    yields check name, check module
+    """
+    checkers_dir = os.path.dirname(checkers.__file__)
+    mod_names = [name for _, name, _ in pkgutil.iter_modules([checkers_dir])]
+    for name in mod_names:
+        mod = importlib.import_module("checkers.{0}".format(name))
 
-    def bash(self, cmd):
-        self.p = Popen(cmd, shell=True, stdout=PIPE, stdin=PIPE, stderr=PIPE)
-        self.output, self.err = self.p.communicate(input=self.output)
-        return self
-
-    def __str__(self):
-        return self.output.strip().decode(encoding='UTF-8')
-
-    def __nonzero__(self):
-        return bool(str(self))
+        # Does the module have a "run" function
+        if isinstance(getattr(mod, 'run', None), types.FunctionType):
+            # has a run method, yield it
+            yield name, mod
 
 
 def title_print(msg):
@@ -60,57 +62,6 @@ def title_print(msg):
     print(bar)
 
 
-def python_files_for_commit():
-    "Get all python files that are staged for commit, that are not deleted."
-    files_pattern = '\.py(\..+)?$'
-    return bash((
-        "git diff --cached --name-status | "
-        "grep -E '{files_pattern}' | "
-        "grep -v -E '^D' | "
-        "awk '{{ print ( $(NF) ) }}' "
-    ).format(files_pattern=files_pattern))
-
-
-def python3():
-    "Check to see if python files are py3 compatible"
-    py_files = str(python_files_for_commit())
-    errors = []
-    for py_file in py_files.splitlines():
-        b = bash('python3 -m py_compile {0}'.format(py_file))
-        if b.err:
-            errors.append(b.err)
-    return "\n".join(errors)
-
-
-def pdb():
-    "Look for pdb.set_trace() commands in python files."
-    forbidden = '^[^#"]*pdb.set_trace()'
-    py_files = python_files_for_commit()
-    if not py_files:
-        return
-    files = py_files.bash((
-        "xargs grep --color --with-filename -n "
-        "-e '{forbidden}'"
-    ).format(
-        forbidden=forbidden
-    ))
-    return files
-
-
-def flake8():
-    "Check flake8 errors in the code base."
-    py_files = str(python_files_for_commit())
-    if not py_files:
-        return
-    b = bash("flake8 {0}".format(py_files.replace('\n', ' ')))
-    if b.err:
-        if "command not found" in b.err:
-            return (
-                "flake8 is required for the flake8 plugin.\n"
-                "`pip install flake8` or turn it off in your tox.ini file.")
-    return b
-
-
 def changes_to_stash():
     "Check there are changes to stash"
     return bool(bash('git diff'))
@@ -152,20 +103,6 @@ def get_hook_checks():
     return {}
 
 
-def get_check_function(check_name):
-    """
-    Get the check function being used in tox.ini
-
-    This can be anything on the PYTHONPATH.
-    """
-    try:
-        return globals()[check_name]
-    except KeyError:
-        print("TODO: Implement importing of extensions.")
-
-CHECKS = (pdb, flake8, python3)
-
-
 def main(stash):
     """
     Run the configured code checks.
@@ -177,10 +114,11 @@ def main(stash):
     exit_code = 0
     hook_checks = get_hook_checks()
     with gitstash(stash):
-        for func in CHECKS:
-            name = func.__name__
+        for name, mod in checks():
+
             if hook_checks.get(name, 'on') == 'on':
-                errors = func()
+                errors = mod.run()
+
                 if errors:
                     title_print("Checking {0}".format(name))
                     print(errors)
